import discord
from discord.ext import commands, tasks
import sqlite3
import logging
from datetime import datetime, timedelta, timezone
import google.generativeai as genai
import os

logging.basicConfig(level=logging.INFO)

intents = discord.Intents.default()
intents.message_content = True
intents.members = True
bot = commands.Bot(command_prefix="/", intents=intents)

API_KEY = None
gemini_model = None
servers = None
send_daily_channel_id_list = None
newcomer_channel_id = None
member_remove_channel_id = None
not_reviewed_id = None
welcome_channel_id = None
ALLOWED_ROLE_IDS = None
WHITELISTED_SERVERS = None
TARGET_CHANNEL_ID = None
discord_bot_token = None

class Config:
    def __init__(
        self,
        api_key,
        gemini_model,
        servers,
        send_daily_channel_id_list,
        newcomer_channel_id,
        member_remove_channel_id,
        not_reviewed_id,
        welcome_channel_id,
        allowed_role_ids,
        whitelisted_servers,
        target_channel_id,
        discord_bot_token,
    ):
        self.api_key = api_key
        self.gemini_model = gemini_model
        self.servers = servers
        self.send_daily_channel_id_list = send_daily_channel_id_list
        self.newcomer_channel_id = newcomer_channel_id
        self.member_remove_channel_id = member_remove_channel_id
        self.not_reviewed_id = not_reviewed_id
        self.welcome_channel_id = welcome_channel_id
        self.allowed_role_ids = allowed_role_ids
        self.whitelisted_servers = whitelisted_servers
        self.target_channel_id = target_channel_id
        self.discord_bot_token = discord_bot_token

    def display(self):
        return {
            "API_KEY": self.api_key,
            "gemini_model": self.gemini_model,
            "servers": self.servers,
            "send_daily_channel_id_list": self.send_daily_channel_id_list,
            "newcomer_channel_id": self.newcomer_channel_id,
            "member_remove_channel_id": self.member_remove_channel_id,
            "not_reviewed_id": self.not_reviewed_id,
            "welcome_channel_id": self.welcome_channel_id,
            "ALLOWED_ROLE_IDS": self.allowed_role_ids,
            "WHITELISTED_SERVERS": self.whitelisted_servers,
            "TARGET_CHANNEL_ID": self.target_channel_id,
            "discord_bot_token": self.discord_bot_token,
        }

def initialize_bot(config):
    global API_KEY, gemini_model, servers, send_daily_channel_id_list
    global newcomer_channel_id, member_remove_channel_id, not_reviewed_id
    global welcome_channel_id, ALLOWED_ROLE_IDS, WHITELISTED_SERVERS
    global TARGET_CHANNEL_ID, discord_bot_token

    API_KEY = config.api_key
    gemini_model = config.gemini_model
    servers = config.servers
    send_daily_channel_id_list = config.send_daily_channel_id_list
    newcomer_channel_id = config.newcomer_channel_id
    member_remove_channel_id = config.member_remove_channel_id
    not_reviewed_id = config.not_reviewed_id
    welcome_channel_id = config.welcome_channel_id
    ALLOWED_ROLE_IDS = config.allowed_role_ids
    WHITELISTED_SERVERS = config.whitelisted_servers
    TARGET_CHANNEL_ID = config.target_channel_id
    discord_bot_token = config.discord_bot_token

    genai.configure(api_key=API_KEY)



def init_db(db_name, tables):
    db_path = "./databases/" + db_name
    os.makedirs(os.path.dirname(db_path), exist_ok=True)
    with sqlite3.connect(db_path) as conn:
        cursor = conn.cursor()
        for table_name, table_schema in tables.items():
            cursor.execute(f"CREATE TABLE IF NOT EXISTS {table_name} ({table_schema})")
        conn.commit()

def get_current_time_utc8():
    timestamp = (datetime.now(timezone.utc) + timedelta(hours=8)).strftime(
        "%Y-%m-%d %H:%M:%S"
    )
    print(timestamp)
    return timestamp

def init_db_points(guild_id):
    db_name = "points_" + str(guild_id) + ".db"
    db_path = "./databases/" + db_name
    os.makedirs(os.path.dirname(db_path), exist_ok=True)
    conn = sqlite3.connect(db_path)
    cursor = conn.cursor()
    cursor.execute(
        """
    CREATE TABLE IF NOT EXISTS users (
        user_id TEXT PRIMARY KEY,
        user_name TEXT,
        join_date TEXT,
        points INTEGER DEFAULT 0
    )
    """
    )
    cursor.execute(
        """
       CREATE TABLE IF NOT EXISTS transactions (
           transaction_id INTEGER PRIMARY KEY AUTOINCREMENT,
           user_id TEXT,
           points INTEGER,
           reason TEXT,
           timestamp TEXT
       )
       """
    )
    conn.commit()
    conn.close()
    
def run_bot():
    from .bot import bot_run  
    bot_run()  

__all__ = [
    'bot', 'initialize_bot', 'init_db', 'get_current_time_utc8', 'init_db_points', 'Config', 'run_bot'
]



import discord
from discord import app_commands
from discord.ext import commands, tasks
from discord_interactions import InteractionType, InteractionResponseType
from datetime import datetime, timedelta, timezone
import sqlite3
from nana_bot import bot, ALLOWED_ROLE_IDS
import logging


@bot.tree.command(name="pass", description="審核通過")
async def pass_user(interaction: discord.Interaction, member: discord.Member):
    server_id = interaction.guild.id
    if server_id == 1212105433943117844:
        role_id_add = 1212117045890523256
        role_id_remove = 1275643437823168624
        embed = discord.Embed(
            title="歡迎加入",
            description=f"{member.mention} 已通過審核，可以先到 <#1212130394548473927> 打聲招呼，也歡迎到 <#1282901299624677448> 或 <#1282901518265225277> 找Wen或是我(奈奈)聊聊喔!",
        )
    elif server_id == 1283632521661124701:
        role_id_add = 1283638437634773072
        role_id_remove = 1284023837692002345
        embed = discord.Embed(
            title="歡迎加入",
            description=f"{member.mention} 已通過審核，可以先到 <#1283649384311164999> 打聲招呼，也歡迎到 <#1285403233916948491> 或 <#1283653144760287252>  找媛媛或是我聊聊喔!",
        )
    if not any(role.id in ALLOWED_ROLE_IDS for role in interaction.user.roles):
        embed = discord.Embed(title="ERROR錯誤!!!", description=f"你沒有權限使用此指令")
        await interaction.response.send_message(embed=embed, ephemeral=True)
        return
    if (
        interaction.channel.id != 1212120624122826812
        and interaction.channel.id != 1283646860409573387
    ):
        embed = discord.Embed(
            title="睜大妳的眼睛看看這是啥頻道吧你",
            description=f"此指令只能在指定的頻道中使用，睜大你的眼睛看看這裡是啥頻道。",
        )
        await interaction.response.send_message(embed=embed, ephemeral=True)
        return
    db_name = "analytics_server_" + str(interaction.guild.id) + ".db"
    conn = sqlite3.connect("./databases/" + db_name)
    cursor = conn.cursor()
    cursor.execute(
        """
       CREATE TABLE IF NOT EXISTS reviews (
           review_id INTEGER PRIMARY KEY AUTOINCREMENT,
           user_id TEXT,
           review_date TEXT
       )
       """
    )
    cursor.execute(
        """
       INSERT INTO reviews (user_id, review_date) 
       VALUES (?, ?)
       """,
        (str(member.id), datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")),
    )
    conn.commit()
    conn.close()

    role_add = interaction.guild.get_role(role_id_add)
    await member.add_roles(role_add)

    role_remove = interaction.guild.get_role(role_id_remove)
    await member.remove_roles(role_remove)
    await interaction.response.send_message(embed=embed)



import discord
from search_engine_tool_vito1317 import google, bing, yahoo
from discord import app_commands
from discord.ext import commands, tasks
from discord_interactions import InteractionType, InteractionResponseType
from datetime import datetime, timedelta, timezone
import sqlite3
from nana_bot import bot
import logging
engines = [
    app_commands.Choice(name='Google', value='google'),
    app_commands.Choice(name='yahoo', value='yahoo')
]
@bot.tree.command(name='search', description='搜尋網路上的酷東西')
@app_commands.describe(engine='選擇搜索引擎', text='輸入搜索內容')
@app_commands.choices(engine=engines)
async def search(interaction: discord.Interaction, engine: str, text: str):
    allowed_engines = ['google', 'bing', 'yahoo']
    await interaction.response.defer()
    if engine not in allowed_engines:
        await interaction.followup.send(f"無效的搜尋引擎：{engine}。請使用以下之一：{', '.join(allowed_engines)}")
        return
    server_id = interaction.guild.id
    timestamp = (datetime.utcnow() + timedelta(hours=8)).strftime('%Y-%m-%d %H:%M:%S')

    def search(engine, query):
        try:
            if engine == 'google':
                results = google.search(query, 'request')
            elif engine == 'bing':
                results = bing.search(query)
            elif engine == 'yahoo':
                results = yahoo.search(query, 'request')

        except Exception as e:
            print(e)
            return []

        content = ""
        for result in results:
            content += f"標題: {result['title']}\n"
            content += f"鏈接: {result['href']}\n"
            content += f"摘要: {result['abstract']}\n\n"
        return content

    content = search(engine, text)

    with sqlite3.connect("./databases/"+f'messages_chat_{server_id}.db') as conn:
        c = conn.cursor()
        c.execute("INSERT INTO message (user, content, timestamp) VALUES (?, ?, ?)",
                  ("奈奈", content, timestamp))
        conn.commit()
    max_length = 2000
    chunks = [content[i:i + max_length] for i in range(0, len(content), max_length)]

    for chunk in chunks:
        embed = discord.Embed(title="查詢結果: " + text, description=chunk,
                              color=discord.Color.green())
        await interaction.followup.send(embed=embed)


import unittest
from nana_bot import Config, initialize_bot, get_current_time_utc8, init_db_points
import os

class TestNanaBot(unittest.TestCase):
    
    def setUp(self):
        # 配置測試參數
        self.config = Config(
            api_key="Test Gemini API Key",
            gemini_model="gemini-1.5-pro-002",
            servers=["test_server_id"],
            send_daily_channel_id_list=["test_send_daily_channel_id"],
            newcomer_channel_id=["test_newcomer_channel_id"],
            member_remove_channel_id=["test_member_remove_channel_id"],
            not_reviewed_id=["test_not_reviewed_id"],
            welcome_channel_id=["test_welcome_channel_id"],
            allowed_role_ids={"test_allowed_role_ids"},
            whitelisted_servers={"test_server_id": "Test Server"},
            target_channel_id=["test_target_channel_id"],
            discord_bot_token="Test discord bot token"
        )

    def test_initialize_bot(self):
        # 測試機器人初始化
        bot = initialize_bot(self.config)
        
        self.assertIsNotNone(bot)
        self.assertEqual(bot.api_key, "Test Gemini API Key")
        self.assertEqual(bot.gemini_model, "gemini-1.5-pro-002")
        self.assertEqual(bot.servers, ["test_server_id"])
        self.assertEqual(bot.send_daily_channel_id_list, ["test_send_daily_channel_id"])
        self.assertEqual(bot.newcomer_channel_id, ["test_newcomer_channel_id"])
        self.assertEqual(bot.member_remove_channel_id, ["test_member_remove_channel_id"])
        self.assertEqual(bot.not_reviewed_id, ["test_not_reviewed_id"])
        self.assertEqual(bot.welcome_channel_id, ["test_welcome_channel_id"])
        self.assertEqual(bot.allowed_role_ids, {"test_allowed_role_ids"})
        self.assertEqual(bot.whitelisted_servers, {"test_server_id": "Test Server"})
        self.assertEqual(bot.target_channel_id, ["test_target_channel_id"])
        self.assertEqual(bot.discord_bot_token, "Test discord bot token")

    def test_get_current_time_utc8(self):
        current_time = get_current_time_utc8()
        self.assertIsNotNone(current_time)
        
    def test_init_db_points(self):
        init_db_points("test_guild_id")
        db_path = "./databases/points_test_guild_id.db"
        self.assertTrue(os.path.exists(db_path))

if __name__ == "__main__":
    unittest.main()



from nana_bot import Config, initialize_bot, run_bot


user_config = Config(
    api_key="User's Gemini API Key",
    gemini_model="gemini-1.5-pro-002",
    servers=["user_servers_id"],
    send_daily_channel_id_list=["user_send_daily_channel_id"],
    newcomer_channel_id=["user_newcomer_channel_id"],
    member_remove_channel_id=["user_member_remove_channel_id"],
    not_reviewed_id=["user_not_reviewed_id"],
    welcome_channel_id=["user_welcome_channel_id"],
    allowed_role_ids={"user_ALLOWED_ROLE_IDS"},
    whitelisted_servers={"User's Server ID": "Server 1"},
    target_channel_id=["user_TARGET_CHANNEL_ID"],
    discord_bot_token="Your Discord Bot Token"
)

initialize_bot(user_config)
run_bot()
